@@ -1,4 +1,4 @@
-# $Id: Ingredient.pm 60 2006-05-04 00:04:23Z sky $
+# $Id: Ingredient.pm 506 2008-06-30 17:52:14Z ykerherve $
 
 package Ingredient;
 use strict;
@@ -6,8 +6,7 @@ use base qw( Data::ObjectDriver::BaseObject );
 
 use Carp ();
 use Data::ObjectDriver::Driver::DBI;
-use Data::ObjectDriver::Driver::Cache::Cache;
-use Cache::Memory;
+use Data::ObjectDriver::Driver::Cache::RAM;
 
 our %IDs;
 
@@ -15,11 +14,11 @@ __PACKAGE__->install_properties({
     columns => [ 'id', 'recipe_id', 'name', 'quantity' ],
     datasource => 'ingredients',
     primary_key => [ 'recipe_id', 'id' ],
-    driver      => Data::ObjectDriver::Driver::Cache::Cache->new(
-        cache => Cache::Memory->new,
+    driver      => Data::ObjectDriver::Driver::Cache::RAM->new(
         fallback => Data::ObjectDriver::Driver::DBI->new(
             dsn      => 'dbi:SQLite:dbname=global.db',
             pk_generator => \&generate_pk,
+            reuse_dbh => 1,
         ),
         pk_generator => \&generate_pk,
     ),
