@@ -1,4 +1,4 @@
-# $Id: Recipe.pm 232 2006-08-05 23:27:32Z btrott $
+# $Id: Recipe.pm 506 2008-06-30 17:52:14Z ykerherve $
 
 package Recipe;
 use strict;
@@ -12,16 +12,22 @@ __PACKAGE__->install_properties({
     primary_key => 'recipe_id',
     driver => Data::ObjectDriver::Driver::DBI->new(
         dsn      => 'dbi:SQLite:dbname=global.db',
+        reuse_dbh => 1,
     ),
 });
 
+my %drivers;
 __PACKAGE__->has_partitions(
     number => 2,
     get_driver => sub {
-        return Data::ObjectDriver::Driver::DBI->new(
-            dsn => 'dbi:SQLite:dbname=cluster' . shift() . '.db',
-            @_,
-        ),
+        my $cluster = shift;
+        my $driver = $drivers{$cluster} ||= 
+            Data::ObjectDriver::Driver::DBI->new(
+                dsn => 'dbi:SQLite:dbname=cluster' . $cluster . '.db',
+                reuse_dbh => 1,
+                @_,
+            );
+        return $driver;
     },
 );
 
